@@ -47,7 +47,7 @@ using namespace bpp;
 
 using namespace std;
 
-MafBlock* AlignmentFilterMafIterator::analyseCurrentBlock_() throw (Exception)
+MafBlock* AlignmentFilterMafIterator::analyseCurrentBlock_()
 {
   if (blockBuffer_.size() == 0) {
     //Else there is no more block in the buffer, we need to parse more:
@@ -125,7 +125,7 @@ MafBlock* AlignmentFilterMafIterator::analyseCurrentBlock_() throw (Exception)
         }
         bool test = (sumEnt / static_cast<double>(windowSize_)) > maxEnt_;
         if (relative_) {
-          double propGap = static_cast<double>(sumGap) / static_cast<double>(nr * nc);
+          double propGap = static_cast<double>(sumGap) / static_cast<double>(nr * windowSize_);
           test = test && (propGap > maxPropGap_);
         } else {
           test = test && (sumGap > maxGap_);
@@ -170,7 +170,7 @@ MafBlock* AlignmentFilterMafIterator::analyseCurrentBlock_() throw (Exception)
       }
       bool test = (sumEnt / static_cast<double>(windowSize_)) > maxEnt_;
       if (relative_) {
-        double propGap = static_cast<double>(sumGap) / static_cast<double>(nr);
+        double propGap = static_cast<double>(sumGap) / static_cast<double>(nr * windowSize_);
         test = test && (propGap > maxPropGap_);
       } else {
         test = test && (sumGap > maxGap_);
@@ -271,7 +271,7 @@ MafBlock* AlignmentFilterMafIterator::analyseCurrentBlock_() throw (Exception)
   return block;
 }
 
-MafBlock* AlignmentFilter2MafIterator::analyseCurrentBlock_() throw (Exception)
+MafBlock* AlignmentFilter2MafIterator::analyseCurrentBlock_()
 {
   if (blockBuffer_.size() == 0) {
     //Else there is no more block in the buffer, we need to parse more:
