@@ -40,6 +40,7 @@ knowledge of the CeCILL license and that you accept its terms.
 #include "MafStatistics.h"
 #include <Bpp/Seq/Container/SequenceContainerTools.h>
 #include <Bpp/Seq/Container/VectorSiteContainer.h>
+#include <Bpp/Seq/Container/SiteContainerTools.h>
 #include <Bpp/Seq/SiteTools.h>
 
 //From bpp-core:
@@ -64,36 +65,12 @@ void PairwiseDivergenceMafStatistics::compute(const MafBlock& block)
     result_.setValue(100. - SequenceTools::getPercentIdentity(*seqs1[0], *seqs2[0], true));
 }
 
-vector<string> CharacterCountsMafStatistics::getSupportedTags() const
-{
-  vector<string> tags;
-  for (int i = 0; i < static_cast<int>(alphabet_->getSize()); ++i) {
-    tags.push_back(alphabet_->intToChar(i));
-  }
-  tags.push_back("Gap");
-  tags.push_back("Unresolved");
-
-  return tags;
-}
-
-void CharacterCountsMafStatistics::compute(const MafBlock& block)
-{
-  std::map<int, int> counts;
-  SequenceContainerTools::getCounts(block.getAlignment(), counts); 
-  for (int i = 0; i < static_cast<int>(alphabet_->getSize()); ++i) {
-    result_.setValue(alphabet_->intToChar(i), counts[i]);
-  }
-  result_.setValue("Gap", counts[alphabet_->getGapCharacterCode()]);
-  double countUnres = 0;
-  for (map<int, int>::iterator it = counts.begin(); it != counts.end(); ++it) {
-    if (alphabet_->isUnresolved(it->first))
-      countUnres += it->second;
-  }
-  result_.setValue("Unresolved", countUnres);
-}
-
 SiteContainer* AbstractSpeciesSelectionMafStatistics::getSiteContainer_(const MafBlock& block)
 {
+  if (noSpeciesMeansAllSpecies_ && species_.size() == 0) {
+    return new VectorSiteContainer(block.getAlignment());
+  }
+  //Otherwise, we select species:
   VectorSiteContainer* alignment = new VectorSiteContainer(block.getAlignment().getAlphabet());
   for (size_t i = 0; i < species_.size(); ++i) {
     if (block.hasSequenceForSpecies(species_[i])) {
@@ -135,6 +112,35 @@ vector<SiteContainer*> AbstractSpeciesMultipleSelectionMafStatistics::getSiteCon
   return alignments;
 }
 
+vector<string> CharacterCountsMafStatistics::getSupportedTags() const
+{
+  vector<string> tags;
+  for (int i = 0; i < static_cast<int>(alphabet_->getSize()); ++i) {
+    tags.push_back(alphabet_->intToChar(i));
+  }
+  tags.push_back("Gap");
+  tags.push_back("Unresolved");
+
+  return tags;
+}
+
+void CharacterCountsMafStatistics::compute(const MafBlock& block)
+{
+  std::map<int, int> counts;
+  unique_ptr<SiteContainer> sites(getSiteContainer_(block));
+  SequenceContainerTools::getCounts(*sites, counts); 
+  for (int i = 0; i < static_cast<int>(alphabet_->getSize()); ++i) {
+    result_.setValue(alphabet_->intToChar(i), counts[i]);
+  }
+  result_.setValue("Gap", counts[alphabet_->getGapCharacterCode()]);
+  double countUnres = 0;
+  for (map<int, int>::iterator it = counts.begin(); it != counts.end(); ++it) {
+    if (alphabet_->isUnresolved(it->first))
+      countUnres += it->second;
+  }
+  result_.setValue("Unresolved", countUnres);
+}
+
 vector<string> SiteFrequencySpectrumMafStatistics::getSupportedTags() const
 {
   vector<string> tags;
@@ -156,7 +162,7 @@ void SiteFrequencySpectrumMafStatistics::compute(const MafBlock& block)
   int state;
   bool hasOutgroup = (outgroup_ != "");
   bool isAnalyzable;
-  auto_ptr<SiteContainer> alignment;
+  unique_ptr<SiteContainer> alignment;
   const Sequence* outgroupSeq = 0;
   if (hasOutgroup) {
     isAnalyzable = (block.hasSequenceForSpecies(outgroup_) && block.getNumberOfSequences() > 1);
@@ -259,7 +265,7 @@ vector<string> FourSpeciesPatternCountsMafStatistics::getSupportedTags() const
 void FourSpeciesPatternCountsMafStatistics::compute(const MafBlock& block)
 {
   counts_.assign(6, 0);
-  auto_ptr<SiteContainer> alignment(getSiteContainer_(block));
+  unique_ptr<SiteContainer> alignment(getSiteContainer_(block));
   if (alignment->getNumberOfSequences() == 4) {
     unsigned int nbIgnored = 0;
     for (size_t i = 0; i < block.getNumberOfSites(); ++i) {
@@ -298,28 +304,50 @@ vector<string> SiteMafStatistics::getSupportedTags() const
   vector<string> tags;
   tags.push_back("NbWithoutGap");
   tags.push_back("NbComplete");
+  tags.push_back("NbConstant");
+  tags.push_back("NbBiallelic");
+  tags.push_back("NbTriallelic");
+  tags.push_back("NbQuadriallelic");
   tags.push_back("NbParsimonyInformative");
   return tags;
 }
 
 void SiteMafStatistics::compute(const MafBlock& block)
 {
-  auto_ptr<SiteContainer> alignment(getSiteContainer_(block));
+  unique_ptr<SiteContainer> alignment(getSiteContainer_(block));
   unsigned int nbNg = 0;
   unsigned int nbCo = 0;
   unsigned int nbPi = 0;
+  unsigned int nbP1 = 0;
+  unsigned int nbP2 = 0;
+  unsigned int nbP3 = 0;
+  unsigned int nbP4 = 0;
   if (alignment->getNumberOfSequences() > 0) {
     for (size_t i = 0; i < alignment->getNumberOfSites(); ++i) {
       if (!SiteTools::hasGap(alignment->getSite(i)))
         nbNg++;
-      if (SiteTools::isComplete(alignment->getSite(i)))
+      if (SiteTools::isComplete(alignment->getSite(i))) {
         nbCo++;
+        map<int, size_t> counts;
+        SiteTools::getCounts(alignment->getSite(i), counts);
+        switch (counts.size()) {
+          case 1: nbP1++; break;
+          case 2: nbP2++; break;
+          case 3: nbP3++; break;
+          case 4: nbP4++; break;
+          default: throw Exception("The impossible happened. Probably a distortion in the Minkowski space.");
+        }
+      }
       if (SiteTools::isParsimonyInformativeSite(alignment->getSite(i)))
         nbPi++;
     }
   }
   result_.setValue("NbWithoutGap", nbNg);
   result_.setValue("NbComplete", nbCo);
+  result_.setValue("NbConstant", nbP1);
+  result_.setValue("NbBiallelic", nbP2);
+  result_.setValue("NbTriallelic", nbP3);
+  result_.setValue("NbQuadriallelic", nbP4);
   result_.setValue("NbParsimonyInformative", nbPi);
 }
 
@@ -445,33 +473,62 @@ vector<string> SequenceDiversityMafStatistics::getSupportedTags() const
   vector<string> tags;
   tags.push_back("NbSeggregating");
   tags.push_back("WattersonTheta");
+  tags.push_back("TajimaPi");
+  tags.push_back("TajimaD");
   return tags;
 }
 
 void SequenceDiversityMafStatistics::compute(const MafBlock& block)
 {
-  auto_ptr<SiteContainer> alignment(getSiteContainer_(block));
-  unsigned int nbSeg = 0;
-  unsigned int nbTot = 0;
-  if (alignment->getNumberOfSequences() > 0) {
-    for (size_t i = 0; i < alignment->getNumberOfSites(); ++i) {
-      const Site& site = alignment->getSite(i);
-      if (SiteTools::isComplete(site)) {
-        nbTot++;
-        if (!SiteTools::isConstant(site))
-          nbSeg++;
-      }
+  unique_ptr<SiteContainer> alignment(getSiteContainer_(block));
+  //Get only complete sites:
+  unique_ptr<SiteContainer> alignment2(SiteContainerTools::getCompleteSites(*alignment));
+  double S = 0;
+  size_t nbTot = alignment2->getNumberOfSites();
+  size_t n = alignment2->getNumberOfSequences();
+  if (n > 0) {
+    for (size_t i = 0; i < alignment2->getNumberOfSites(); ++i) {
+      const Site& site = alignment2->getSite(i);
+      if (!SiteTools::isConstant(site))
+        S++;
     }
   }
-  double wt = 0;
-  if (nbSeg > 0) {
-    size_t n = alignment->getNumberOfSequences();
-    double hf = 0;
-    for (double i = 1; i < n; ++i)
-      hf += 1. / i;
-    wt = static_cast<double>(nbSeg) / (static_cast<double>(nbTot) * hf);
+
+  double a1 = 0;
+  double a2 = 0;
+  double dn = static_cast<double>(n);
+  for (double i = 1; i < dn; ++i) {
+    a1 += 1. / i;
+    a2 += 1. / (i * i);
   }
-  result_.setValue("NbSeggregating", nbSeg);
+  double wt = S / (static_cast<double>(nbTot) * a1);
+  double b1 = (dn + 1) / (3 * (dn - 1));
+  double b2 = 2 * (dn * dn + dn + 3) / (9 * dn * (dn - 1));
+  double c1 = b1 - 1. / a1;
+  double c2 = b2 - (dn + 2) / (a1 * dn) + a2 / (a1 * a1);
+  double e1 = c1 / a1;
+  double e2 = c2 / (a1 * a1 + a2);
+  
+  //Compute pairwise heterozigocity:
+  double pi = 0;
+  for (size_t i = 0; i < n - 1; ++i) {
+    for (size_t j = i + 1; j < n; ++j) {
+      pi += SiteContainerTools::computeSimilarity(
+          alignment2->getSequence(i),
+          alignment2->getSequence(j),
+          true,
+          SiteContainerTools::SIMILARITY_ALL,
+          false);
+    }
+  }
+  pi /= static_cast<double>((n - 1) * n / 2);
+
+  //Compute Tajima's D:
+  double tajd = static_cast<double>(nbTot) * (pi - wt) / sqrt(e1 * S + e2 * S * (S - 1));
+
+  result_.setValue("NbSeggregating", S);
   result_.setValue("WattersonTheta", wt);
+  result_.setValue("TajimaPi", pi);
+  result_.setValue("TajimaD", tajd);
 }
 
